# -*- coding: utf-8 -*-
from django.http import HttpResponse
from rest_framework.viewsets import GenericViewSet
from rest_framework import mixins
from djangorestfilemanager.models import File
from djangorestfilemanager.filters import FileFilter
from djangorestfilemanager.permissions import FilePermissions
from djangorestfilemanager.app_settings import FileSerializer, FileViewSerializer
import logging

logger = logging.getLogger(__name__)


class FileModelViewSet(mixins.CreateModelMixin, mixins.RetrieveModelMixin, GenericViewSet):
    queryset = File.objects.all()
    serializer_class = FileSerializer
    filter_class = FileFilter
    search_fields = ['name', 'uuid']
    ordering_fields = ['name', 'uuid', 'username', 'origin', 'type']
    permission_classes = (FilePermissions,)

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return FileViewSerializer
        return FileSerializer

    def perform_create(self, serializer):
        user = self.request.user
        filename = serializer.validated_data.get('file').name
        instance = serializer.save(username=user.username, name=filename)
        logger.info('- File uuid: {}, uploaded by user {}'.format(instance.uuid, user.username))

    def retrieve(self, request, *args, **kwargs):
        obj = self.get_object()
        self.check_object_permissions(request, obj)
        response = HttpResponse(obj.file, content_type='application/octet-stream')
        response['Content-Disposition'] = 'attachment; filename=%s' % obj.name
        logger.info('- File uuid: {}, downloaded by user {}'.format(obj.uuid, request.user.username))
        return response



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjangoRestFileManagerConfig(AppConfig):
    name = 'djangorestfilemanager'
    verbose_name = _('Django Rest File Manager Config')



from django.urls import include
from django.conf.urls import url
from rest_framework import routers

from djangorestfilemanager.views import FileModelViewSet

router = routers.SimpleRouter()
router.register(r'(?P<version>(v1))/files', FileModelViewSet)

urlpatterns = router.urls



from djangorestfilemanager.models import File
from rest_framework import serializers
import logging

from djangorestfilemanager.settings import STATUS_CHOICES, DEFAULT_CHOICE, DEFAULT_SHAREABLE

logger = logging.getLogger(__name__)


class DefaultFileSerializer(serializers.ModelSerializer):
    status = serializers.ChoiceField(choices=STATUS_CHOICES, default=DEFAULT_CHOICE)
    share = serializers.BooleanField(default=DEFAULT_SHAREABLE)

    class Meta:
        model = File
        fields = (
            'url', 'uuid', 'file', 'name', 'username', 'creation_date', 'last_mod_date', 'permission', 'type',
            'origin', 'share', 'status'
        )
        read_only_fields = ('uuid', 'name', 'username', 'creation_date', 'last_mod_date')

    def to_representation(self, instance):
        return DefaultFileViewSerializer(instance=instance).data


class DefaultFileViewSerializer(serializers.ModelSerializer):
    class Meta:
        model = File
        fields = ('uuid',)
        read_only_fields = ('uuid',)



from django.conf import settings
from importlib import import_module
from six import string_types

from djangorestfilemanager.serializers import DefaultFileSerializer
from djangorestfilemanager.serializers import DefaultFileViewSerializer


def import_callable(path_or_callable):
    if hasattr(path_or_callable, '__call__'):
        return path_or_callable
    else:
        assert isinstance(path_or_callable, string_types)
        package, attr = path_or_callable.rsplit('.', 1)
        return getattr(import_module(package), attr)


serializers = getattr(settings, 'DJANGO_REST_FILE_MANAGER_SERIALIZERS', {})

FileSerializer = import_callable(
    serializers.get('FILE_SERIALIZER', DefaultFileSerializer)
)
FileViewSerializer = import_callable(
    serializers.get('FILE_VIEW_SERIALIZER', DefaultFileViewSerializer)
)



# Generated by Django 3.0.3 on 2020-03-02 08:47

from django.db import migrations, models
import djangorestfilemanager.models


class Migration(migrations.Migration):

    dependencies = [
        ('djangorestfilemanager', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='file',
            field=models.FileField(storage=djangorestfilemanager.models.RestFileManagerStorage(location='media'), upload_to=djangorestfilemanager.models.set_storage_file_dir, verbose_name='File'),
        ),
        migrations.AlterField(
            model_name='file',
            name='origin',
            field=models.CharField(blank=True, default='API', max_length=3, null=True, verbose_name='Origin'),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import time
import uuid

from django.contrib.postgres.fields import JSONField
from django.db import models
from django.template.defaultfilters import slugify
from django.conf import settings
import os
from storages.backends.s3boto3 import S3Boto3Storage
from django.core.files.storage import default_storage
from djangorestfilemanager.settings import UPLOADS_DIR


class RestFileManagerStorage(S3Boto3Storage):

    def get_available_name(self, name, max_length=None):
        now = time.time()
        stamp = datetime.datetime.fromtimestamp(now).strftime('%Y-%m-%d-%H-%M-%S-%f')
        return slugify('{}_{}'.format(name, stamp))


def get_file_system_storage():
    if getattr(settings, 'S3_STORAGE'):
        return RestFileManagerStorage(location=getattr(settings, 'AWS_LOCATION', 'files'))
    else:
        return default_storage


def set_storage_file_dir(instance, filename):
    return os.path.join(UPLOADS_DIR, instance.type, filename)


class File(models.Model):
    MASSIVE_FILE_TYPE = 'MFT'

    uuid = models.UUIDField(
        default=uuid.uuid4,
        primary_key=True
    )
    file = models.FileField(
        verbose_name='File',
        storage=get_file_system_storage(),
        upload_to=set_storage_file_dir,
        blank=False,
        null=True
    )
    name = models.CharField(
        verbose_name='Name',
        max_length=255
    )
    username = models.CharField(
        'Username',
        max_length=100
    )
    creation_date = models.DateTimeField(
        verbose_name='Creation date',
        auto_now_add=True,
    )
    last_mod_date = models.DateTimeField(
        verbose_name='Creation date',
        auto_now=True,
    )
    origin = models.CharField(
        verbose_name='Origin',
        blank=True,
        null=True,
        max_length=3,
        default='API'
    )
    type = models.CharField(
        verbose_name='Type',
        blank=True,
        null=True,
        max_length=10,
        default='file'
    )
    permission = models.CharField(
        verbose_name='Permission',
        max_length=200,
        blank=True,
        null=True,
        default=''
    )
    share = models.BooleanField(
        verbose_name='Share',
        default=True
    )
    status = models.CharField(
        'Status',
        max_length=3
    )
    data = JSONField(default=dict, blank=True, null=True)

    class Meta:
        verbose_name = 'File'
        verbose_name_plural = 'Files'

    def __str__(self):
        return '{}-{}'.format(self.name, self.type)

    def to_dict(self):
        return {
            'uuid': str(self.uuid),
            'name': self.name,
            'origin': self.origin,
            'type': self.type,
            'username': self.username
        }
